      subroutine ffs_qp(u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a scalar array
c           u=u1*(gvm*P- + gvp*P+)*u2
c                         *(-g_mn + q_m*q_n/mv2)/(q^2-mv2).
c
c Input: 
c        g - a fermion-fermion-scala coupling.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 23 March, 2005; modified 31 March, 2005, 3 April, 2005, 22 March, 2006,
c                          27 April, 2006, 16 February, 2007,
c                          13 May (interface changed again)
c                              20 May, 2007 (sign of p changed)
c 29 Aug. 2019: quadrple precision four momenta implemented,
c 8 Oct. 2019: bug in declaration of g(2) corrected
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer :: n1,n2,n,i1,i2,i
      integer, parameter :: dp=kind(1.d0),qp=kind(1.q0)

      real(dp) :: mass,wdth
      real(qp), dimension(0:3) :: p1,p2,p

      complex(dp) :: g(2)
      complex(qp) :: u1(4,n1),u2(4,n2),u(1,n),den

      p=p1+p2
c      p=-p1-p2 ! Virtual scalar boson is always incoming to the vertex.

      den=cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2-mass**2,mass*wdth,qp)

      i=0

      do i2=1,n2
       do i1=1,n1
        i=i+1
        u(1,i)=(g(2)*(u1(1,i1)*u2(1,i2)+u1(2,i1)*u2(2,i2))
     &       +g(1)*(u1(3,i1)*u2(3,i2)+u1(4,i1)*u2(4,i2)))/den
       end do ! i1
      end do ! i2

      end subroutine ffs_qp



