      subroutine pppvmd_qp(ig,g,u0,p0,u1,p1,u2,p2,u,mat)
c----------------------------------------------------------------------c
c Returns a matrix element for a quartic vector meson-pi0-pi+pi- 
c coupling with scalars u0,u1,u2 and a vector u
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        g - a quartic vector meson-pi0-pi+pi- coupling,
c        u0,u1,u2 - scalars of pi0,pi+,pi-,
c        u - vector,
c        p0,p1,p2 - momenta.
c Output: mat - a matrix element.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 15 Oct., 2014; modified on 28/31 Oct., 2014;
c 19 March, 2015 (call to subroutine runcoupl implemented)
c 9 March, 2020 (quadruple precision version written)
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      integer :: ig
      real(qp), dimension(0:3) :: p0,p1,p2,p
      complex(dp) :: g,rg
      complex(qp) :: u0(1),u1(1),u2(1),u(0:3),mat,r(0:3)

      p=-p0-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl_qp(ig,p,rg)
      end if

      r=(/cmplx(-(p0(1)*(p1(2)*p2(3)-p1(3)*p2(2))
     &           -p0(2)*(p1(1)*p2(3)-p1(3)*p2(1))
     &           +p0(3)*(p1(1)*p2(2)-p1(2)*p2(1))),0.q0,qp),
     &    cmplx( -p0(0)*(p1(2)*p2(3)-p1(3)*p2(2))
     &           +p0(2)*(p1(0)*p2(3)-p1(3)*p2(0))
     &           -p0(3)*(p1(0)*p2(2)-p1(2)*p2(0)),0.q0,qp),
     &    cmplx(  p0(0)*(p1(1)*p2(3)-p1(3)*p2(1))
     &           -p0(1)*(p1(0)*p2(3)-p1(3)*p2(0))
     &           +p0(3)*(p1(0)*p2(1)-p1(1)*p2(0)),0.q0,qp),
     &    cmplx( -p0(0)*(p1(1)*p2(2)-p1(2)*p2(1))
     &           +p0(1)*(p1(0)*p2(2)-p1(2)*p2(0))
     &           -p0(2)*(p1(0)*p2(1)-p1(1)*p2(0)),0.q0,qp)/)

      mat=rg*u0(1)*u1(1)*u2(1)*(r(0)*u(0)
     &                        -r(1)*u(1)-r(2)*u(2)-r(3)*u(3))

      end subroutine pppvmd_qp
