      subroutine weightfactor_qp(iwgt,wgt,arg,p,mass,factor)
c----------------------------------------------------------------------c
c Returns a complex factor=wgt*e^(i*arg)*f(s) that is included in the
c mixing diagrams
c
c Input: iwgt=1,2,3,... - options of the factor, supplement your own 
c                         option below,
c        p - four momentum of a particle,
c        mass, wdth - mass and width of a particle.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 12 March, 2015; modified on 
c 19 March, 2015: a module inprms added
c 6 March, 2020: 
c quadruple precision version written
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)
      real(dp), parameter :: drad=3.141592653589793238d0/180.d0
      integer :: iwgt
      real(qp) :: p(0:3)
      real(dp) :: wgt,arg,mass,s,rfac
      complex(dp) :: factor

      if (iwgt == 1) then
       rfac=wgt
      else if (iwgt == 2) then
       s=p(0)**2-p(1)**2-p(2)**2-p(3)**2
       if (s > 0.d0) then
        rfac=wgt*sqrt(s)/mass
       else
        rfac=0.d0
       end if
      else if (iwgt == 3) then
       s=p(0)**2-p(1)**2-p(2)**2-p(3)**2
       if (s > 0.d0) then
        rfac=wgt*s/mass**2
       else
        rfac=0.d0
       end if
      else
        write(7,*)'weightfactor.f: iwgt=',iwgt,' is not implemented'
      end if

      factor=cmplx(rfac*cos(arg*drad),rfac*sin(arg*drad),dp)

      end subroutine weightfactor_qp
